<?php

/**
 * @file
 * Contains a TagFormatterPlugin
 *
 * @license GPL v2 http://www.fsf.org/licensing/licenses/gpl.html
 * @author Chris Skene chris at previousnext dot com dot au
 */

namespace Drupal\ghost_formatters\Plugins;

use Drupal\ghost\Formatters\Plugins\BaseFormatterPlugin;
use Drupal\ghost\Formatters\Plugins\FormatterPluginInterface;

$plugin = array(
  'name' => 'ghost_tag',
  'class' => '\Drupal\ghost_formatters\Plugins\TagFormatterPlugin',
  'title' => t('Tag Formatter'),
);

/**
 * Class TagFormatterPlugin
 * @package Drupal\formatters\Plugins\Formatters
 */
class TagFormatterPlugin extends BaseFormatterPlugin implements FormatterPluginInterface {

  /**
   * Get the machine name for the formatter.
   *
   * @return string
   *   The machine name for the formatter.
   */
  public function getMachineName() {
    return 'ghost_tag';
  }

  /**
   * Get info for a plugin.
   *
   * @return array
   *   The plugin's info.
   */
  public function info() {

    return array(
      'label' => t('Tag'),
      'field types' => array('text'),
      'settings'  => array(
        'element' => 'div',
        'link' => FALSE,
      ),
    );
  }

  /**
   * Get a settings form.
   *
   * @param mixed $field
   *   The field structure being configured.
   * @param array $instance
   *   The instance structure being configured.
   * @param string $view_mode
   *   The view mode being configured.
   * @param array $form
   *   The (entire) configuration form array, which will usually have no use
   *   here.
   * @param array $form_state
   *   The form state of the (entire) configuration form.
   * @param array $settings
   *   (Optional) An array of settings.
   *
   * @return array
   *   The form elements.
   */
  public function settingsForm($field, $instance, $view_mode, $form, &$form_state, array $settings = array()) {
    $element = array();

    if (isset($settings['element']) && array_key_exists($settings['element'], $this->getAllowedTags())) {
      $element_setting = $settings['element'];
    }
    else {
      $element_setting = reset($this->getAllowedTags());
    }

    $element['element'] = array(
      '#type'           => 'select',
      '#title'          => t('Element'),
      '#description'    => t('Select the HTML element to wrap the field in'),
      '#default_value'  => $element_setting,
      '#options'        => $this->getAllowedTags(),
    );
    $element['link'] = array(
      '#type'           => 'checkbox',
      '#title'          => t('Link to entity'),
      '#description'    => t("Whether to link this to its entity."),
      '#default_value'  => $settings['link'],
    );

    return $element;
  }

  /**
   * Return a short summary for the current formatter settings of an instance.
   *
   * If an empty result is returned, the formatter is assumed to have no
   * configurable settings, and no UI will be provided to display a
   * settings form.
   *
   * @param mixed $field
   *   The field structure.
   * @param array $instance
   *   The instance structure.
   * @param string $view_mode
   *   The view mode for which a settings summary is requested.
   * @param array $settings
   *   The forms settings.
   *
   * @return NULL|string
   *   The summary or NULL
   */
  public function settingsSummary($field, $instance, $view_mode, $settings) {

    $summary = t('Wrapped in a @element element', array(
      '@element' => $settings['element'],
    ));

    if ($settings['link'] == TRUE) {
      $summary .= t(', linked to entity.');
    }

    return $summary;
  }

  /**
   * Callback for hook_field_formatter_view().
   *
   * Builds a renderable array for a field value.
   *
   * @param string $entity_type
   *   The type of entity.
   * @param object $entity
   *   The entity being displayed.
   * @param array $field
   *   The field structure.
   * @param array $instance
   *   The field instance.
   * @param string $langcode
   *   The language associated with items.
   * @param array $items
   *   Array of values for this field.
   * @param array $display
   *   The display settings to use, as found in the 'display' entry of
   *   instance definitions. The array notably contains the following keys and
   *   values:
   *   - type: The name of the formatter to use
   *   - settings: The array of formatter settings.
   *
   * @return array
   *   A renderable array for the $items, as an array of child elements
   *   keyed by numeric indexes starting from 0.
   */
  public function view($entity_type, $entity, $field, $instance, $langcode, $items, $display) {
    $element = array();

    foreach ($items as $delta => $item) {
      $value = field_filter_xss($item['value']);
      if ($display['settings']['link'] == TRUE) {
        $uri = entity_uri($entity_type, $entity);
        if (!empty($uri['path'])) {
          $value = l(field_filter_xss($item['value']), $uri['path']);
        }
      }

      $element[$delta] = array(
        '#theme' => 'ghost_tag',
        '#tag' => $display['settings']['element'],
        '#value' => $value,
      );
    }
    return $element;
  }

  /**
   * Get allowed tags.
   *
   * @return array
   *   An array of allowed tags.
   */
  protected function getAllowedTags() {

    $defaults = array('div', 'span', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6');

    $allowed = variable_get('ghost_formatters_allowed_tags', $defaults);

    $final = array();
    if (!empty($allowed)) {
      foreach ($allowed as $tag) {
        $final[$tag] = $tag;
      }
    }

    return $final;
  }
}
